require 'minitest/autorun'
require 'image_comparator'
require 'fileutils'

class ImageComparatorTest < Minitest::Test

  def setup
    create_tmp_folder_if_required
    @image_comparator = ImageComparator.new
  end

  def teardown
    clean_images
  end

  def clean_images
    FileUtils.rm_rf Dir.glob('./tmp/*.png')
  end

  def create_tmp_folder_if_required
    folder = 'tmp'
    FileUtils::mkdir_p folder unless File.directory?(folder)
  end

  def test_compare_pixel_perfect_almost
    @image_comparator.define_images_to_compare('./images/almost-the-same-1.png', './images/almost-the-same-2.png')
    compare = @image_comparator.compare
    refute(compare, 'Images are not equal')
  end

  def test_compare_pixel_perfect
    @image_comparator.define_images_to_compare('./images/base.png', './images/base-ok.png')
    compare = @image_comparator.compare
    assert(compare, 'Images are not equal')
  end

  def test_not_compare_pixel_perfect
    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok.png')
    compare = @image_comparator.compare
    refute(compare, 'Images are equal')
  end

  def test_calculate_diff_image_ok
    path_res = './tmp/ok_diff.png'
    expected_fingerprint = 0

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-ok.png')
    current_fingerprint = @image_comparator.calculate_diff_image(path_res)
    fingerprint_diff_file = Phashion::Image.new(path_res).fingerprint

    assert_equal(expected_fingerprint, current_fingerprint, 'Method fingerprint is not the expceted one')
    assert_equal(expected_fingerprint, fingerprint_diff_file, 'Diff file fingerprint is not the expceted one')
  end

  def test_calculate_diff_image_fail
    path_res = './tmp/fail_diff.png'
    expected_fingerprint = 5450986487329114955

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok.png')
    current_fingerprint = @image_comparator.calculate_diff_image(path_res)
    fingerprint_diff_file = Phashion::Image.new(path_res).fingerprint

    assert_equal(expected_fingerprint, current_fingerprint, 'Method fingerprint is not the expceted one')
    assert_equal(expected_fingerprint, fingerprint_diff_file, 'Diff file fingerprint is not the expceted one')
  end

  def test_compare_and_generate_diff_ok
    @image_comparator.define_images_to_compare('./images/base.png', './images/base-ok.png')
    compare = @image_comparator.compare_and_generate_diff
    assert(compare, 'Images are not equal')
  end

  def test_compare_and_generate_diff_nok
    expected_fingerprint = 5450986487329114955
    path_res = './tmp/fail_diff.png'

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok.png')
    compare = @image_comparator.compare_and_generate_diff(path_res)
    fingerprint_diff_file = Phashion::Image.new(path_res).fingerprint

    refute(compare, 'Images are equal')
    assert_equal(expected_fingerprint, fingerprint_diff_file, 'Diff file fingerprint is not the expceted one')
  end

  def test_compare_area_include
    origin_coord = {'x':828, 'y':293}
    end_coord = {'x':1173, 'y':688}
    area = [{'origin': origin_coord, 'end': end_coord, 'exclude': false}]

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok-areas.png')
    @image_comparator.set_areas(area)
    compare = @image_comparator.compare

    assert(compare, 'Images are not equals')
  end

  def test_compare_area_exclude
    origin_coord = {'x':828, 'y':293}
    end_coord = {'x':1173, 'y':688}
    area = [{'origin': origin_coord, 'end': end_coord, 'exclude': true}]

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok.png')
    @image_comparator.set_areas(area)
    compare = @image_comparator.compare

    assert(compare, 'Images are not equals')
  end

  def test_compare_several_areas_include
    origin_coord = {'x':828, 'y':293}
    end_coord = {'x':1173, 'y':688}
    area1 = {'origin': origin_coord, 'end': end_coord, 'exclude': false}
    origin_coord2 = {'x':0, 'y':0}
    end_coord2 = {'x':828, 'y':293}
    area2 = {'origin': origin_coord2, 'end': end_coord2, 'exclude': false}
    areas = [area1, area2]

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok-areas.png')
    @image_comparator.set_areas(areas)
    compare = @image_comparator.compare

    refute(compare, 'Images are compares')
  end

  def test_compare_several_areas_exclude
    origin_coord = {'x':828, 'y':293}
    end_coord = {'x':1173, 'y':688}
    area1 = {'origin': origin_coord, 'end': end_coord, 'exclude': true}
    origin_coord2 = {'x':0, 'y':0}
    end_coord2 = {'x':828, 'y':293}
    area2 = {'origin': origin_coord2, 'end': end_coord2, 'exclude': true}
    areas = [area1, area2]

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok-areas.png')
    @image_comparator.set_areas(areas)
    compare = @image_comparator.compare

    assert(compare, 'Images are not equals')
  end

  def test_compare_several_areas_mix_not_overlapping
    origin_coord = {'x':828, 'y':293}
    end_coord = {'x':1173, 'y':688}
    area1 = {'origin': origin_coord, 'end': end_coord, 'exclude': false}
    origin_coord2 = {'x':0, 'y':0}
    end_coord2 = {'x':828, 'y':293}
    area2 = {'origin': origin_coord2, 'end': end_coord2, 'exclude': true}
    areas = [area1, area2]

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok-areas.png')
    @image_comparator.set_areas(areas)
    compare = @image_comparator.compare

    assert(compare, 'Images are differents')
  end

  def test_compare_several_areas_mix_overlapping
    origin_coord = {'x':828, 'y':293}
    end_coord = {'x':1173, 'y':688}
    area1 = {'origin': origin_coord, 'end': end_coord, 'exclude': false}
    origin_coord2 = {'x':0, 'y':0}
    end_coord2 = {'x':1000, 'y':490}
    area2 = {'origin': origin_coord2, 'end': end_coord2, 'exclude': true}
    areas = [area1, area2]

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok-areas.png')
    @image_comparator.set_areas(areas)
    compare = @image_comparator.compare

    assert(compare, 'Images are differents')
  end

  def test_compare_and_generate_diff_area_include
    origin_coord = {'x':828, 'y':293}
    end_coord = {'x':1173, 'y':688}
    area = [{'origin': origin_coord, 'end': end_coord, 'exclude': false}]
    path_res = './tmp/ok_diff.png'

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-ok.png')
    @image_comparator.set_areas(area)
    compare = @image_comparator.compare_and_generate_diff(path_res)

    assert(compare, 'Images are differents')
  end

  def test_compare_and_generate_diff_area_exclude
    origin_coord = {'x':828, 'y':293}
    end_coord = {'x':1173, 'y':688}
    area = [{'origin': origin_coord, 'end': end_coord, 'exclude': true}]
    path_res = './tmp/ok_diff.png'

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok.png')
    @image_comparator.set_areas(area)
    compare = @image_comparator.compare_and_generate_diff(path_res)

    assert(compare, 'Images are differents')
  end

  def test_compare_and_generate_diff_several_areas_mix_overlapping
    origin_coord = {'x':828, 'y':293}
    end_coord = {'x':1173, 'y':688}
    area1 = {'origin': origin_coord, 'end': end_coord, 'exclude': false}
    origin_coord2 = {'x':0, 'y':0}
    end_coord2 = {'x':1000, 'y':490}
    area2 = {'origin': origin_coord2, 'end': end_coord2, 'exclude': true}
    areas = [area1, area2]
    path_res = './tmp/ok_diff.png'

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok-areas.png')
    @image_comparator.set_areas(areas)
    compare = @image_comparator.compare_and_generate_diff(path_res)

    assert(compare, 'Images are differents')
  end

  def test_compare_and_generate_diff_area_include_fail
    origin_coord = {'x':828, 'y':293}
    end_coord = {'x':1173, 'y':688}
    area = [{'origin': origin_coord, 'end': end_coord, 'exclude': false}]
    path_res = './tmp/nok_diff.png'
    expected_fingerprint = 5450986487329114955

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok.png')
    @image_comparator.set_areas(area)
    compare = @image_comparator.compare_and_generate_diff(path_res)
    refute(compare, 'Images are equal')

    fingerprint_diff_file = Phashion::Image.new(path_res).fingerprint
    assert_equal(expected_fingerprint, fingerprint_diff_file, 'Diff file fingerprint is not the expected one')
  end

  def test_compare_and_generate_diff_area_exclude_fail
    origin_coord = {'x':0, 'y':0}
    end_coord = {'x':1000, 'y':490}
    area = [{'origin': origin_coord, 'end': end_coord, 'exclude': true}]
    path_res = './tmp/nok_diff.png'
    expected_fingerprint = 5452075012271155531

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok.png')
    @image_comparator.set_areas(area)
    compare = @image_comparator.compare_and_generate_diff(path_res)
    refute(compare, 'Images are equal')

    fingerprint_diff_file = Phashion::Image.new(path_res).fingerprint
    assert_equal(expected_fingerprint, fingerprint_diff_file, 'Diff file fingerprint is not the expected one')
  end

  def test_compare_and_generate_diff_several_areas_mix_overlapping_fail
    origin_coord = {'x':828, 'y':293}
    end_coord = {'x':1173, 'y':688}
    area1 = {'origin': origin_coord, 'end': end_coord, 'exclude': true}
    origin_coord2 = {'x':0, 'y':0}
    end_coord2 = {'x':1000, 'y':490}
    area2 = {'origin': origin_coord2, 'end': end_coord2, 'exclude': false}
    areas = [area1, area2]
    path_res = './tmp/nok_diff.png'
    expected_fingerprint = 54086765383280

    @image_comparator.define_images_to_compare('./images/base.png', './images/base-nok-areas.png')
    @image_comparator.set_areas(areas)
    compare = @image_comparator.compare_and_generate_diff(path_res)
    refute(compare, 'Images are equal')

    fingerprint_diff_file = Phashion::Image.new(path_res).fingerprint
    assert_equal(expected_fingerprint, fingerprint_diff_file, 'Diff file fingerprint is not the expected one')
  end
end